@@ -13,10 +13,11 @@ from django import forms
 from django.contrib import admin
 from django.urls import reverse
 from django.utils.html import format_html
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from admin_tools_stats.app_label_renamer import AppLabelRenamer
-from admin_tools_stats.models import CriteriaToStatsM2M, DashboardStats, DashboardStatsCriteria
+from admin_tools_stats.models import (CriteriaToStatsM2M, DashboardStats,
+                                      DashboardStatsCriteria)
 
 AppLabelRenamer(native_app_label=u'admin_tools_stats', app_label=_('Admin Tools Stats')).main()
 
@@ -89,12 +90,45 @@ class DashboardStatsAdmin(admin.ModelAdmin):
     Allows the administrator to view and modify certain attributes
     of a DashboardStats.
     """
-    list_display = ('id', 'graph_key', 'analytics_link', 'graph_title', 'model_name', 'distinct', 'type_operation_field_name',
-                    'is_visible', 'created_date', 'date_field_name', 'operation_field_name', 'default_chart_type')
-    list_filter = ['created_date']
+    fieldsets = (
+        (None, {
+            'fields': (
+                'graph_key',
+                'graph_title',
+                ('model_app_name', 'model_name', 'date_field_name'),
+                ('operation_field_name', 'distinct'),
+                ('user_field_name', 'show_to_users'),
+                ('allowed_type_operation_field_name', 'type_operation_field_name'),
+                ('allowed_chart_types', 'default_chart_type'),
+                ('allowed_time_scales', 'default_time_scale', 'default_time_period'),
+                'y_axis_format',
+                'default_multiseries_criteria',
+                'is_visible',
+            )
+        }),
+    )
+    list_display = (
+        'id',
+        'graph_key',
+        'analytics_link',
+        'graph_title',
+        'model_name',
+        'distinct',
+        'type_operation_field_name',
+        'is_visible',
+        'show_to_users',
+        'created_date',
+        'date_field_name',
+        'operation_field_name',
+        'default_chart_type',
+    )
+    list_filter = [
+        'created_date',
+        'is_visible',
+        'show_to_users',
+    ]
     exclude = ('criteria',)
     inlines = [DashboardStatsCriteriaInline]
-    ordering = ('id', )
     save_as = True
     form = DashboardStatsForm
 
