@@ -8,34 +8,36 @@
 # The Initial Developer of the Original Code is
 # Arezqui Belaid <info@star2billing.com>
 #
-import logging
 import datetime
+import logging
 from collections import OrderedDict
-from datetime import timedelta
 
 from cache_utils.decorators import cached
-
-from dateutil.relativedelta import relativedelta, MO
-from dateutil.rrule import rrule, YEARLY, MONTHLY, WEEKLY, DAILY, HOURLY
-
+from dateutil.relativedelta import MO, relativedelta
+from dateutil.rrule import DAILY, HOURLY, MONTHLY, WEEKLY, YEARLY, rrule
 from django.apps import apps
 from django.conf import settings
 from django.core.exceptions import FieldError, ValidationError
 from django.db import models
 from django.db.models import ExpressionWrapper, Q
-from django.db.models.aggregates import Avg, Count, Max, Min, StdDev, Sum, Variance
+from django.db.models.aggregates import (Avg, Count, Max, Min, StdDev, Sum,
+                                         Variance)
 from django.db.models.fields import DateField
 from django.db.models.functions import Trunc
 from django.db.models.signals import post_save
 from django.dispatch import receiver
-from django.urls import reverse
 from django.utils import timezone
-from django.utils.encoding import force_text
 from django.utils.safestring import mark_safe
-from django.utils.timezone import now
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
+from multiselectfield import MultiSelectField
 
-import jsonfield.fields
+try:
+    if getattr(settings, 'ADMIN_CHARTS_USE_JSONFIELD', True):
+        from django.db.models import JSONField
+    else:
+        from jsonfield.fields import JSONField
+except ImportError:
+    from jsonfield.fields import JSONField
 
 logger = logging.getLogger(__name__)
 
@@ -67,14 +69,16 @@ time_scales = (
     ('days', 'Days'),
     ('weeks', 'Weeks'),
     ('months', 'Months'),
+    ('quarters', 'Quarters'),
     ('years', 'Years'),
 )
-freqs = {
-    'years': YEARLY,
-    'months': MONTHLY,
-    'weeks': WEEKLY,
-    'days': DAILY,
-    'hours': HOURLY
+rrule_freqs = {
+    'years': {'freq': YEARLY},
+    'quarters': {'freq': MONTHLY, 'interval': 3},
+    'months': {'freq': MONTHLY},
+    'weeks': {'freq': WEEKLY},
+    'days': {'freq': DAILY},
+    'hours': {'freq': HOURLY},
 }
 
 
@@ -91,6 +95,9 @@ def truncate(dt, interval):
         return day - relativedelta(weekday=MO(-1))
     elif interval == 'months':
         return datetime.datetime(dt.year, dt.month, 1, tzinfo=dt.tzinfo)
+    elif interval == 'quarters':
+        qmonth = dt.month - (dt.month - 1) % 3
+        return datetime.datetime(dt.year, qmonth, 1, tzinfo=dt.tzinfo)
     elif interval == 'years':
         return datetime.datetime(dt.year, 1, 1, tzinfo=dt.tzinfo)
 
@@ -115,7 +122,7 @@ class DashboardStatsCriteria(models.Model):
         verbose_name=_('criteria name'),
         help_text=_("it needs to be one word unique. Ex. status, yesno"),
     )
-    criteria_fix_mapping = jsonfield.fields.JSONField(
+    criteria_fix_mapping = JSONField(
         null=True, blank=True,
         verbose_name=_("fixed criteria / value"),
         help_text=_("a JSON dictionary of key-value pairs that will be used for the criteria"),
@@ -125,7 +132,7 @@ class DashboardStatsCriteria(models.Model):
         verbose_name=_("dynamic criteria field name"),
         help_text=_("ex. for call records - disposition"),
     )
-    criteria_dynamic_mapping = jsonfield.fields.JSONField(
+    criteria_dynamic_mapping = JSONField(
         null=True, blank=True,
         verbose_name=_("dynamic criteria / value"),
         help_text=_(
@@ -191,39 +198,57 @@ class DashboardStats(models.Model):
     )
     model_app_name = models.CharField(
         max_length=90, verbose_name=_('app name'),
-        help_text=_("ex. auth / dialer_cdr"),
+        help_text=_("ex. <i>auth</i>, <i>dialer_cdr</i>"),
     )
     model_name = models.CharField(
         max_length=90, verbose_name=_('model name'),
-        help_text=_("ex. User"),
+        help_text=_("ex. <i>User</i>"),
     )
     date_field_name = models.CharField(
         max_length=90, verbose_name=_("date field name"),
-        help_text=_("ex. date_joined, invitation__invitation_date"),
+        help_text=_("ex. <i>date_joined</i>, <i>invitation__invitation_date</i>"),
     )
     user_field_name = models.CharField(
         max_length=90, verbose_name=_("user field name"),
         null=True, blank=True,
-        help_text=_("ex. owner, invitation__owner"),
+        help_text=_(
+            "User field that will limit chart data only for currently logged user (if not superuser)<br/>"
+            "ex. owner, <i>invitation__owner</i>",
+        ),
+    )
+    show_to_users = models.BooleanField(
+        verbose_name=_("show to ordinary users"),
+        null=False, blank=False,
+        default=False,
+        help_text=_("Be carefull and test if it still doesn't expose sensitive data"),
     )
     operation_field_name = models.CharField(
         max_length=90, verbose_name=_("Operate field name"),
         null=True, blank=True,
-        help_text=_("The field you want to aggregate, ex. amount, salaries__total_income"),
+        help_text=_(
+            "The field you want to aggregate, ex. <i>amount</i>, <i>salaries__total_income</i>.<br/>"
+            "Can contain multiple fields divided by comma.",
+        ),
     )
     distinct = models.BooleanField(
         default=False,
         null=False,
         blank=True,
         help_text=_(
-            "Note: Distinct is supported only for Count, Sum, Avg and 'Avgerage count per active model instance'.<br/>"
-            "Django>=3.0 is needed for distinct Sum and Avg."
+            "Note: Distinct is supported only for <i>Count</i>, <i>Sum</i>, "
+            "<i>Avg</i> and <i>'Avgerage count per active model instance'</i>.<br/>"
+            "Django>=3.0 is needed for distinct <i>Sum</i> and <i>Avg</i>.",
         ),
     )
     type_operation_field_name = models.CharField(
         max_length=90, verbose_name=_("Choose Type operation"),
         null=True, blank=True, choices=operation,
-        help_text=_("choose the type operation what you want to aggregate, ex. Sum"),
+        help_text=_("Choose the type operation what you want to aggregate."),
+    )
+    allowed_type_operation_field_name = MultiSelectField(
+        max_length=1000, verbose_name=_("Choose Type operation"),
+        null=True, blank=True, choices=operation,
+        help_text=_("More than one selected field will display chooser on the chart."),
     )
     default_chart_type = models.CharField(
         max_length=90,
@@ -233,6 +258,15 @@ class DashboardStats(models.Model):
         choices=chart_types,
         default='discreteBarChart',
     )
+    allowed_chart_types = MultiSelectField(
+        max_length=1000,
+        verbose_name=_("Allowed chart types"),
+        null=True,
+        blank=True,
+        choices=chart_types,
+        default=list(zip(*chart_types))[0],
+        help_text=_("More than one selected field will display chooser on the chart."),
+    )
     default_time_period = models.PositiveIntegerField(
         verbose_name=_("Default period"),
         help_text=_("Number of days"),
@@ -248,12 +282,19 @@ class DashboardStats(models.Model):
         choices=time_scales,
         max_length=90,
     )
+    allowed_time_scales = MultiSelectField(
+        max_length=1000,
+        verbose_name=_("Allowed time scales"),
+        choices=time_scales,
+        default=list(zip(*time_scales))[0],
+        help_text=_("More than one selected field will display chooser on the chart."),
+    )
     y_axis_format = models.CharField(
         max_length=90,
         verbose_name=_("Y axis format"),
         help_text=_(
             "Format of Y axis."
-            "<a href='https://github.com/d3/d3-format' target='_blank'>See description of possible values</a>."
+            "<a href='https://github.com/d3/d3-format' target='_blank'>See description of possible values</a>.",
         ),
         null=True,
         blank=True,
@@ -278,12 +319,21 @@ class DashboardStats(models.Model):
         verbose_name = _("dashboard stats")
         verbose_name_plural = _("dashboard stats")
 
+    def allowed_type_operation_field_name_choices(self):
+        return (op for op in operation if op[0] in self.allowed_type_operation_field_name)
+
+    def allowed_time_scales_choices(self):
+        return (ts for ts in time_scales if ts[0] in self.allowed_time_scales)
+
+    def allowed_chart_types_choices(self):
+        return (cht for cht in chart_types if cht[0] in self.allowed_chart_types)
+
     def get_model(self):
         return apps.get_model(self.model_app_name, self.model_name)
 
-    def get_operation_field(self):
+    def get_operation_field(self, operation):
         query = self.get_model().objects.all().query
-        return query.resolve_ref(self.operation_field_name).field
+        return query.resolve_ref(operation).field
 
     def get_date_field(self):
         query = self.get_model().objects.all().query
@@ -303,8 +353,9 @@ class DashboardStats(models.Model):
             errors['model_name'] = str(e)
 
         try:
-            if model and self.operation_field_name:
-                self.get_operation_field()
+            for operation in self.get_operations_list():
+                if model and operation:
+                    self.get_operation_field(operation)
         except FieldError as e:
             errors['operation_field_name'] = str(e)
 
@@ -317,13 +368,26 @@ class DashboardStats(models.Model):
         raise ValidationError(errors)
         return super(DashboardStats, self).clean(*args, **kwargs)
 
-    def get_operation(self, dkwargs=None):
-        operation = {
+    def get_operations_list(self):
+        if self.operation_field_name:
+            return self.operation_field_name.split(',')
+        return []
+
+    def get_operation(self, operation_choice, operation_field_choice, dkwargs=None):
+        if not operation_choice:
+            operation_choice = self.type_operation_field_name or 'Count'
+        if not operation_field_choice:
+            operations = self.get_operations_list()
+            operation_field_choice = operations[0] if operations else 'id'
+        if not operation_field_choice:
+            self.operation_field_name = 'id'
+
+        operations = {
             'AvgCountPerInstance': lambda field_name, distinct, dkwargs: ExpressionWrapper(
                 1.0 *
                 Count(field_name, distinct=distinct, filter=dkwargs) /
                 Count('id', distinct=True, filter=Q(**{field_name + "__isnull": False})),
-                output_field=models.FloatField()
+                output_field=models.FloatField(),
             ),
             'Count': lambda field_name, distinct, dkwargs: Count(field_name, distinct=distinct, filter=dkwargs),
             'Sum': lambda field_name, distinct, dkwargs: Sum(field_name, distinct=distinct, filter=dkwargs),
@@ -333,15 +397,15 @@ class DashboardStats(models.Model):
             'Min': lambda field_name, distinct, dkwargs: Min(field_name, filter=dkwargs),
             'Variance': lambda field_name, distinct, dkwargs: Variance(field_name, filter=dkwargs),
         }
-        return operation[self.type_operation_field_name](self.operation_field_name, self.distinct, dkwargs)
+        return operations[operation_choice](operation_field_choice, self.distinct, dkwargs)
 
-    def get_time_series(self, dynamic_criteria, all_criteria, request, time_since, time_until, interval):
+    def get_time_series(self, dynamic_criteria, all_criteria, user, time_since, time_until, operation_choice, operation_field_choice, interval):
         """ Get the stats time series """
         model_name = apps.get_model(self.model_app_name, self.model_name)
         kwargs = {}
         dynamic_kwargs = []
-        if request and not request.user.is_superuser and self.user_field_name:
-            kwargs[self.user_field_name] = request.user
+        if not user.has_perm('admin_tools_stats.view_dashboardstats') and self.user_field_name:
+            kwargs[self.user_field_name] = user
         for m2m in all_criteria:
             criteria = m2m.criteria
             # fixed mapping value passed info kwargs
@@ -365,7 +429,9 @@ class DashboardStats(models.Model):
 
                     for dynamic_value in dynamic_values:
                         try:
-                            criteria_value = m2m.get_dynamic_choices(time_since, time_until)[dynamic_value]
+                            criteria_value = m2m.get_dynamic_choices(
+                                time_since, time_until, operation_choice, operation_field_choice, user,
+                            )[dynamic_value]
                         except KeyError:
                             criteria_value = 0
                         if isinstance(criteria_value, (list, tuple)):
@@ -383,21 +449,23 @@ class DashboardStats(models.Model):
         if not dynamic_kwargs:
             dynamic_kwargs = [None]
 
-        for dkwargs in dynamic_kwargs:
-            i += 1
-            if not self.type_operation_field_name:
-                self.type_operation_field_name = 'Count'
-            if not self.operation_field_name:
-                self.operation_field_name = 'id'
-
-            aggregate_dict['agg_%i' % i] = self.get_operation(dkwargs)
+        operations = self.get_operations_list()
+        if operations and len(operations) > 1 and dynamic_criteria['select_box_operation_field'] == '':
+            for operation in operations:
+                i += 1
+                aggregate_dict['agg_%i' % i] = self.get_operation(operation_choice, operation)
+        else:
+            for dkwargs in dynamic_kwargs:
+                i += 1
+                aggregate_dict['agg_%i' % i] = self.get_operation(operation_choice, operation_field_choice, dkwargs)
 
         # TODO: maybe backport values_list support back to django-qsstats-magic and use it again for the query
         time_range = {'%s__range' % self.date_field_name: (time_since, time_until)}
         qs = model_name.objects
         qs = qs.filter(**time_range)
         qs = qs.filter(**kwargs)
-        qs = qs.annotate(d=Trunc(self.date_field_name, interval))
+        kind = interval[:-1]
+        qs = qs.annotate(d=Trunc(self.date_field_name, kind))
         qs = qs.values_list('d')
         qs = qs.order_by('d')
         qs = qs.annotate(**aggregate_dict)
@@ -411,21 +479,21 @@ class DashboardStats(models.Model):
             criteria = None
         return criteria
 
-    def get_multi_time_series(self, configuration, time_since, time_until, interval, request=None):
+    def get_multi_time_series(self, configuration, time_since, time_until, interval, operation_choice, operation_field_choice, user):
         current_tz = timezone.get_current_timezone()
-        time_since_tz = current_tz.localize(time_since)
-        time_until_tz = current_tz.localize(time_until).replace(hour=23, minute=59)
+        time_since_tz = time_since.astimezone(current_tz)
+        time_until_tz = time_until.astimezone(current_tz).replace(hour=23, minute=59)
 
         configuration = configuration.copy()
         series = {}
         all_criteria = self.criteriatostatsm2m_set.all()  # Outside of get_time_series just for performance reasons
         m2m = self.get_multi_series_criteria(configuration)
-        if m2m and m2m.criteria.dynamic_criteria_field_name:
-            choices = m2m.get_dynamic_choices(time_since_tz, time_until_tz)
 
-            serie_map = {}
-            names = []
-            values = []
+        values = []
+        names = []
+        operations = self.get_operations_list()
+        if m2m and m2m.criteria.dynamic_criteria_field_name:
+            choices = m2m.get_dynamic_choices(time_since_tz, time_until_tz, operation_choice, operation_field_choice, user)
             for key, name in choices.items():
                 if key != '':
                     if isinstance(name, (list, tuple)):
@@ -433,83 +501,48 @@ class DashboardStats(models.Model):
                     names.append(name)
                     values.append(key)
             configuration['select_box_dynamic_' + str(m2m.id)] = values
-            serie_map = self.get_time_series(configuration, all_criteria, request, time_since_tz, time_until_tz, interval)
-            for tv in serie_map:
-                time = tv[0]
-                if time not in series:
-                    series[time] = OrderedDict()
-                i = 0
-                for name in names:
-                    i += 1
-                    series[time][name] = tv[i]
+        elif len(operations) > 1 and configuration['select_box_operation_field'] == '':
+            names = operations
         else:
-            serie = self.get_time_series(configuration, all_criteria, request, time_since_tz, time_until_tz, interval)
-            for time, value in serie:
-                series[time] = {'': value}
-            names = {'': ''}
+            names = ['']
+
+        serie_map = {}
+        serie_map = self.get_time_series(
+            configuration, all_criteria, user, time_since_tz, time_until_tz, operation_choice, operation_field_choice, interval,
+        )
+        for tv in serie_map:
+            time = tv[0]
+            if time not in series:
+                series[time] = {}
+            i = 0
+            for name in names:
+                i += 1
+                series[time][name] = tv[i]
 
         # fill with zeros where the records are missing
         start = truncate(time_since, interval)
 
-        dates = list(rrule(freq=freqs[interval], dtstart=start, until=time_until))
+        dates = list(rrule(**rrule_freqs[interval], dtstart=start, until=time_until))
         for time in dates:
             if self.get_date_field().__class__ == DateField:
                 time = time.date()
             elif settings.USE_TZ:
-                time = current_tz.localize(time)
+                time = time.astimezone(current_tz)
 
             if time not in series:
-                series[time] = OrderedDict()
+                series[time] = {}
             for key in names:
                 if key not in series[time]:
                     series[time][key] = 0
         return series
 
-    def get_control_form(self):
+    def get_control_form_raw(self, user=None):
+        from .forms import ChartSettingsForm
+        return ChartSettingsForm(self, user, auto_id=False)
+
+    def get_control_form(self, user=None):
         """ Get content of the ajax control form """
-        temp = ''
-        for i in self.criteriatostatsm2m_set.filter(use_as='chart_filter').order_by('order'):
-            dy_map = i.get_dynamic_choices()
-            if dy_map:
-                temp += i.criteria.criteria_name + ': <select class="chart-input dynamic_criteria_select_box" name="select_box_dynamic_%i" >' % i.id
-                temp += '<option value="">-------</option>'
-                for key, name in dy_map.items():
-                    if isinstance(name, (list, tuple)):
-                        name = name[1]
-                    selected_str = 'selected=selected' if key == i.default_option else ''
-                    temp += '<option value="%s" %s>%s</option>' % (key, selected_str, name)
-                temp += '</select>'
-
-        temp += '<input type="hidden" class="hidden_graph_key" name="graph_key" value="%s">' % self.graph_key
-
-        multiple_series = self.criteriatostatsm2m_set.filter(use_as='multiple_series')
-        if multiple_series.exists():
-            temp += 'Divide: <select class="chart-input select_box_multiple_series" name="select_box_multiple_series" >'
-            temp += '<option class="chart-input" value="">-------</option>'
-            for serie in multiple_series.order_by('order').all():
-                selected_str = 'selected=selected' if serie == serie.stats.default_multiseries_criteria else ''
-                temp += '<option class="chart-input" value="%s" %s>%s</option>' % (serie.id, selected_str, serie.criteria.criteria_name)
-            temp += '</select>'
-
-        temp += 'Scale: <select class="chart-input select_box_interval" name="select_box_interval" >'
-        for interval, interval_name in time_scales:
-            selected_str = 'selected=selected' if interval == self.default_time_scale else ''
-            temp += '<option class="chart-input" value="' + interval + '" ' + selected_str + '>' + interval_name + '</option>'
-        temp += '</select>'
-
-        temp += 'Since: <input class="chart-input select_box_date_since" type="date" name="time_since" value="%s">' % \
-            (now() - timedelta(days=self.default_time_period)).strftime('%Y-%m-%d')
-        temp += 'Until: <input class="chart-input select_box_date_since" type="date" name="time_until" value="%s">' % \
-            now().strftime('%Y-%m-%d')
-
-        temp += 'Chart: <select class="chart-input select_box_chart_type" name="select_box_chart_type" >'
-        for chart_type_str, chart_type_name in chart_types:
-            selected_str = 'selected=selected' if chart_type_str == self.default_chart_type else ''
-            temp += '<option class="chart-input" value="' + chart_type_str + '" ' + selected_str + '>' + chart_type_name + '</option>'
-        temp += '</select>'
-        temp += f"&nbsp;<a href='{reverse('chart-analytics')}?show={self.graph_key}' target='_blank'>analytics</a>"
-
-        return mark_safe(force_text(temp))
+        return self.get_control_form_raw(user).as_p()
 
     def __str__(self):
         return u"%s" % self.graph_key
@@ -585,7 +618,10 @@ class CriteriaToStatsM2M(models.Model):
 
     # The slef argument is here just because of this bug: https://github.com/infoscout/django-cache-utils/issues/19
     @cached(60 * 5)
-    def _get_dynamic_choices(self, slef, time_since=None, time_until=None, count_limit=None):
+    def _get_dynamic_choices(
+            self, slef, time_since=None, time_until=None, count_limit=None,
+            operation_choice=None, operation_field_choice=None, user=None,
+    ):
         model = self.stats.get_model()
         field_name = self.get_dynamic_criteria_field_name()
         if self.criteria.criteria_dynamic_mapping:
@@ -620,13 +656,18 @@ class CriteriaToStatsM2M(models.Model):
                     date_filters['%s__lte' % self.stats.date_field_name] = end_time
                 choices_queryset = model.objects.filter(
                         **date_filters,
-                    ).values_list(
+                    )
+                if user and not user.has_perm('admin_tools_stats.view_dashboardstats'):
+                    if not self.stats.user_field_name:
+                        raise Exception("User field must be defined to enable charts for non-superusers")
+                    choices_queryset = choices_queryset.filter(**{self.stats.user_field_name: user})
+                choices_queryset = choices_queryset.values_list(
                         field_name,
                         flat=True,
                     ).distinct()
                 if count_limit:
                     choices_queryset = choices_queryset.annotate(
-                        f_count=self.stats.get_operation(),
+                        f_count=self.stats.get_operation(operation_choice, operation_field_choice),
                     ).order_by(
                         '-f_count',
                     )
@@ -642,7 +683,7 @@ class CriteriaToStatsM2M(models.Model):
                 )
                 if count_limit:
                     choices.update(
-                        [('other', ([i for i in other_choices_queryset], 'other'))]
+                        [('other', ([i for i in other_choices_queryset], 'other'))],
                     )
                     choices.move_to_end('other', last=False)
                 return choices
@@ -650,11 +691,11 @@ class CriteriaToStatsM2M(models.Model):
     def __str__(self):
         return f"{self.stats.graph_title} - {self.criteria.criteria_name}"
 
-    def get_dynamic_choices(self, time_since=None, time_until=None):
+    def get_dynamic_choices(self, time_since=None, time_until=None, operation_choice=None, operation_field_choice=None, user=None):
         if not self.choices_based_on_time_range:
             time_since = None
             time_until = None
-        choices = self._get_dynamic_choices(self, time_since, time_until, self.count_limit)
+        choices = self._get_dynamic_choices(self, time_since, time_until, self.count_limit, operation_choice, operation_field_choice, user)
         return choices
 
 
