@@ -10,5 +10,6 @@ class TestKid(models.Model):
     age = models.IntegerField()
     bio = models.TextField()
     wanted_games_qtd = models.BigIntegerField()
-    birthday = models.DateField()
-    appointment = models.DateTimeField()
+    birthday = models.DateField(null=True)
+    appointment = models.DateTimeField(null=True)
+    author = models.ForeignKey('auth.User', on_delete=models.CASCADE, default=None, null=True, blank=True)
